require 'fileutils'

module Shacho
  class Account
    def self.create(options)
      # TODO Throw error for incorrect params
      name = options.first
      filename = account_folder(name)
      if !File.exists?(filename) || !File.directory?(filename)
        FileUtils.mkdir_p(filename)
      end
      update_credentials(name)
      Key.generate(name)
    end

    def self.use(options)
      # TODO Throw error for incorrect params
      # TODO Refactor
      name = options.first
      Runner.error account_not_found(name) if !exists?(name)

      credential_file = "#{account_folder(name)}/credentials"
      heroku_credentials = "#{HEROKU_PREFIX}/credentials"
      FileUtils.ln_sf credential_file, heroku_credentials
      Key.use(name)
    end

    def self.list(options)
      # TODO Throw error for incorrect params
      accounts.each do |account|
        account_name = File.basename(account)
        puts account_name if !(["..", "."].include?(account))
      end
    end

    def self.edit(options)
      # TODO Throw error for incorrect params
      # TODO Refactor
      name = options.first
      Runner.error account_not_found(name) if !exists?(name)
      update_credentials(name)
    end

    def self.remove(options)
      # TODO Throw error for incorrect params
      # TODO Refactor
      name = options.first
      Runner.error account_not_found(name) if !exists?(name)

      # TODO Account for a 'corrupted' ~/.heroku/credentials
      #  i.e. pointing to deleted account
      FileUtils.rm_r account_folder(name)
    end

    def self.clear(options)
      accounts.each do |account|
        remove([File.basename(account)])
      end
    end

    def self.which(options)
      puts current_credentials
    end

    def method_missing(sym, *args, &block)
      Runner.error("Unsupported action.  Run `shacho help` for more information")
    end

    private
    def self.account_folder(name)
      return "#{HEROKU_PREFIX}/accounts/#{name}"
    end

    def self.update_credentials(name)
      credential = Credential.new
      credential.prompt(name)
      credential.write(account_folder(name))
    end

    def self.exists?(name)
      File.exists?(account_folder(name))
    end

    def self.account_not_found(name)
      "Error: '#{name}' not found.  Create account with: shacho create #{name}"
    end

    def self.accounts
      Dir.glob "#{HEROKU_PREFIX}/accounts/*"
    end

    def self.current_credentials
      current_credentials = %x[readlink "#{HEROKU_PREFIX}/credentials"]
      account_dir = %x[dirname "#{current_credentials}"]
      return File.basename(account_dir)
    end
  end
end
